﻿; ------------------------------------------------------------------------------------
; Fast Hashtable Include
; Core taken from: http://www.purebasic-lounge.com/viewtopic.php?t=5464&postdays=0&postorder=asc&start=12
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; Structures & Enumerations
; ------------------------------------------------------------------------------------

Enumeration
  #InitialTableSize = 2503
  #CHASHMAP_TYPE_VALUE
  #CHASHMAP_MAP_OVERFLOW = -1
EndEnumeration

Structure sHASHMAP
  Object.l
  Blocksize.l
  Mapsize.l
  count.l
  *map
EndStructure

Structure sElement
  key.s
  used.l
  Value.i
EndStructure

Structure MapArray
  map.sElement[0]
EndStructure

; ------------------------------------------------------------------------------------
; Internal
; ------------------------------------------------------------------------------------

Macro _DoResize(_State, _Maximum)
  _State>(2*_Maximum/3)
EndMacro

Macro _ResizeTo(_current, _blocksize)
  _current + _blocksize
EndMacro

; Procedure _CreateHash(*pKey.CHARACTER, Mapsize.l)
;   !MOV EAX,5381            ;Startwert hash
;   !MOV EBX,EAX             ;für erste Addition in Schleife
;   !MOV ESI,dword[p.p_pKey] ;oder LEA ESI,[v_pKey], aber wenn p_pKey eh schon da ist...
;   !XOR EDX,EDX             ;EDX auf Null setzen
;   !MOV CL,5                ;sollte so schneller sein
;   !@@:
;   !MOV DL,[ESI]            ;also kein Unicode!
;   !OR DL,DL                ;Test auf Null
;   !JZ @f                   ;ist Null, Sprung vorwärts
;   !SHL EAX,CL              ;SHL = SAL; SAL wird selten angegeben
;   !ADD EAX,EBX
;   !ADD EAX,EDX
;   !MOV EBX,EAX             ;"alten" hash sichern
;   !INC ESI
;   !JMP @b                  ;Sprung rückwärts, nächsten Wert einlesen
;   !@@:
;   !MOV dword[p.p_pKey],esi ;wirklich Pointer erhöhen? Sonst weg
;   !XOR EDX,EDX             ;schneller und richtiger (!) als CDQ ich gehe hier mal von vorzeichenlosen Werten aus
;   !DIV dword[p.v_Mapsize]  ;EAX ist noch hash
;   !MOV EAX,EDX             ;Divisions-Rest
;   ProcedureReturn        ;Gibt Hash in EAX zurück
; EndProcedure

Procedure _CreateHash(*pKey.CHARACTER, Mapsize.l)
  Protected hash.l, quad.q
  hash = 5381
  While *pKey\c
    x + 1
    hash = ((hash<<5) + hash) + *pKey\c
    *pKey + SizeOf(CHARACTER)
  Wend
  quad = hash & $FFFFFFFF
  hash = quad%Mapsize
  ProcedureReturn hash
EndProcedure

Procedure _NextPrime(num)
  Protected prime, i, start
  If Int(num/2)*2 = num ; nur Ungerade Zahlen prüfen
    num + 1
  EndIf
  Repeat
    i = 2
    While i*i<num
      If num%i = 0
        prime = #False
        Break
      Else
        prime = #True
      EndIf
      If i> = 3 ; Ab 3 in 3erschritten prüfen
        i + 3
      Else
        i + 2
      EndIf
    Wend
    If prime = #False
      num + 1
      If num%10 = 5 ; Zahlen die mit 5 enden fliegen raus
        num + 1
      EndIf
    Else
      Break
    EndIf
  ForEver
  ProcedureReturn num
EndProcedure

Procedure _ReDimHashmap(*object.sHASHMAP)
  Protected NewSize.l = _NextPrime(_ResizeTo(*object\Mapsize, *object\Blocksize))
  Protected *pNewMap.MapArray = AllocateMemory(NewSize*SizeOf(sElement))
  Protected *p.MapArray = *object\map, i.l, counter.l, r.l, hash.l
  For i = 0 To *object\Mapsize-1
    If *p\map[i]\used = #True
      hash = _CreateHash(@*p\map[i]\key, NewSize)
      counter = 0 : r = 0
      While counter<NewSize
        If *pNewMap\map[hash]\used = #False
          CopyMemory(*p\map[i], *pNewMap\map[hash], SizeOf(sElement))
          Break
        Else
          counter + 1
          hash + ((counter/2)*(counter/2))*(-1 + (r*2))
          hash%NewSize
          If hash<0 : hash + NewSize : EndIf
          r ! 1
        EndIf
      Wend
    EndIf
  Next i
  FreeMemory(*object\map)
  *object\map = *pNewMap
  *object\Mapsize = NewSize
EndProcedure

; ------------------------------------------------------------------------------------
; Public
; ------------------------------------------------------------------------------------

Procedure HT_AddValue(*object.sHASHMAP, key.s, Value.l)
  Protected counter, r
  Protected *p.MapArray, hash, *e.sElement
  hash = _CreateHash(@key, *object\Mapsize)
  *p = *object\map
  While counter<*object\Mapsize And Not *object\count = *object\Mapsize
    *e = *p\map[hash] ; schneller da der Index nur 1* berechnet werden muss
    With *e
      If \used = #False
        \key = key
        \Value = Value
        \used = #True
        *object\count + 1
        If _DoResize(*object\count, *object\Mapsize)
          _ReDimHashmap(*object)
        EndIf
        ProcedureReturn #True
      Else
        counter + 1
        hash + ((counter/2)*(counter/2))*(-1 + (r*2))
        hash%*object\Mapsize
        If hash<0 : hash + *object\Mapsize : EndIf
        r ! 1
      EndIf
    EndWith
  Wend
  ProcedureReturn #CHASHMAP_MAP_OVERFLOW
EndProcedure

Procedure HT_GetValue(*object.sHASHMAP, key.s)
  Protected counter, r
  Protected *p.MapArray, hash, *e.sElement
  hash = _CreateHash(@key, *object\Mapsize)
  *p = *object\map
  While counter<*object\Mapsize
    *e = *p\map[hash] ; schneller da der Index nur 1* berechnet werden muss
    With *e
      If Not \used
        ProcedureReturn #False
      ElseIf \key = key
        ProcedureReturn \Value
      Else ; // quadratic probe
        counter + 1
        hash + ((counter/2)*(counter/2))*(-1 + (r*2))
        hash%*object\Mapsize
        If hash<0 : hash + *object\Mapsize : EndIf
        r ! 1 ; // Swap bool
      EndIf
    EndWith
  Wend
EndProcedure

Procedure HT_Free(*object.sHASHMAP)
  Protected hash.l, i.l, *ptrStr, *p.MapArray
  *p = *object\map
  FreeMemory(*object\map)
  ProcedureReturn #True
EndProcedure

Procedure HT_New(BlockSize = #InitialTableSize)
  Protected *object.sHASHMAP, i.l, prime.l
  BlockSize = _NextPrime(BlockSize)
  *object = AllocateMemory(SizeOf(sHASHMAP))
  If *object
    *object\Object = #True
    *object\BlockSize = Blocksize
    *object\Mapsize = Blocksize
    *object\map = AllocateMemory(Blocksize*SizeOf(sElement))
    ProcedureReturn *object
  EndIf
EndProcedure


; IDE Options = PureBasic 4.30 (Windows - x86)
; CursorPosition = 68
; FirstLine = 58
; Folding = A5
; EnableXP
; DisableDebugger
; EnableCompileCount = 14
; EnableBuildCount = 0
; EnableExeConstant