; ------------------------------------------------------------------------------------
; EasyCluster Include for PB
; Author: Alexander Aigner
; Easy Cluster DLL (c) Alexander Aigner
; ------------------------------------------------------------------------------------

; EnableExplicit

; Mit #ECluster_IncludePath umgeht man ein kleines Manko das PB mit hirachischen Includes hat.
; Einfach den Pfad eintragen in dem ECluster_Inc.pbi liegt

CompilerIf Defined(ECluster_IncludePath, #PB_Constant) = #Null
  #ECluster_IncludePath = ""
CompilerEndIf

; ------------------------------------------------------------------------------------
; Prft Vorraussetzungen
; ------------------------------------------------------------------------------------

CompilerIf #PB_Compiler_Thread = #Null
  CompilerError "Easy Cluster: please enable threadsafe or throw yourself in front of a train, this gives crumbs"
  End
CompilerEndIf

; ------------------------------------------------------------------------------------
; Includieren aller Parts
; ------------------------------------------------------------------------------------

CompilerSelect #PB_Compiler_OS
  CompilerCase #PB_OS_Windows
    IncludeFile #ECluster_IncludePath + "Parts\Misc.pbi"
    IncludeFile #ECluster_IncludePath + "Parts\Master.pbi"
    IncludeFile #ECluster_IncludePath + "Parts\Slave.pbi"
    IncludeFile #ECluster_IncludePath + "Parts\Cache.pbi"
    IncludeFile #ECluster_IncludePath + "Parts\SQLite.pbi"
  CompilerCase #PB_OS_Linux
    IncludeFile #ECluster_IncludePath + "Parts/Misc.pbi"
    IncludeFile #ECluster_IncludePath + "Parts/Master.pbi"
    IncludeFile #ECluster_IncludePath + "Parts/Slave.pbi"
    IncludeFile #ECluster_IncludePath + "Parts/Cache.pbi"
    IncludeFile #ECluster_IncludePath + "Parts/SQLite.pbi"
  CompilerCase #PB_OS_MacOS
    IncludeFile #ECluster_IncludePath + "Parts/Misc.pbi"
    IncludeFile #ECluster_IncludePath + "Parts/Master.pbi"
    IncludeFile #ECluster_IncludePath + "Parts/Slave.pbi"
    IncludeFile #ECluster_IncludePath + "Parts/Cache.pbi"
    IncludeFile #ECluster_IncludePath + "Parts/SQLite.pbi"
CompilerEndSelect

; ------------------------------------------------------------------------------------
; Initialisieren
; ------------------------------------------------------------------------------------

Procedure ECluster_Init(Dll$ = "Windows\ECluster.dll")
  Protected ECluster_Dll
  ECluster_Dll = OpenLibrary(#PB_Any, DLL$)
  If ECluster_DLL
    ECluster_Slave_Init = GetFunction(ECluster_Dll, "ECluster_Slave_Init")
    ECluster_Slave_DeInit = GetFunction(ECluster_Dll, "ECluster_Slave_DeInit")
    _ECluster_Slave_GetLastError = GetFunction(ECluster_Dll, "ECluster_Slave_GetLastError")
    ECluster_Slave_Pause = GetFunction(ECluster_Dll, "ECluster_Slave_Pause")
    ECluster_Slave_Resume = GetFunction(ECluster_Dll, "ECluster_Slave_Resume")
    ECluster_Slave_FreeStream = GetFunction(ECluster_Dll, "ECluster_Slave_FreeStream")
    ECluster_Slave_CountMasters = GetFunction(ECluster_Dll, "ECluster_Slave_CountMasters")
    ECluster_Slave_IterateMasters = GetFunction(ECluster_Dll, "ECluster_Slave_IterateMasters")
    ECluster_Slave_KickMaster = GetFunction(ECluster_Dll, "ECluster_Slave_KickMaster")
    ECluster_Slave_SetMasterJobLimit = GetFunction(ECluster_Dll, "ECluster_Slave_SetMasterJobLimit")
    ECluster_Slave_EnableStreamTransmission = GetFunction(ECluster_Dll, "ECluster_Slave_EnableStreamTransmission")
    ECluster_Slave_CountMasterJobs = GetFunction(ECluster_Dll, "ECluster_Slave_CountMasterJobs")
    ECluster_Slave_GetMasterIP = GetFunction(ECluster_Dll, "ECluster_Slave_GetMasterIP")
    ECluster_Slave_GetMasterPort = GetFunction(ECluster_Dll, "ECluster_Slave_GetMasterPort")
    ECluster_Slave_GetMasterXtraData = GetFunction(ECluster_Dll, "ECluster_Slave_GetMasterXtraData")
    ECluster_Slave_SetMasterXtraData = GetFunction(ECluster_Dll, "ECluster_Slave_SetMasterXtraData")
    ECluster_Slave_SetMasterJobAccessCheck = GetFunction(ECluster_Dll, "ECluster_Slave_SetMasterJobAccessCheck")
    ECluster_Slave_CountJobs = GetFunction(ECluster_Dll, "ECluster_Slave_CountJobs")
    ECluster_Slave_SetReceiveBlockSize = GetFunction(ECluster_Dll, "ECluster_Slave_SetReceiveBlockSize")
    ECluster_Slave_SetCompressionOptions = GetFunction(ECluster_Dll, "ECluster_Slave_SetCompressionOptions")
    ECluster_Slave_SetCallback = GetFunction(ECluster_Dll, "ECluster_Slave_SetCallback")
    ECluster_Slave_ThreadReceive = GetFunction(ECluster_Dll, "ECluster_Slave_ThreadReceive")
    ECluster_Slave_EnableStatistics = GetFunction(ECluster_Dll, "ECluster_Slave_EnableStatistics")
    ECluster_Slave_AddJob = GetFunction(ECluster_Dll, "ECluster_Slave_AddJob")
    ECluster_Slave_LogEvents = GetFunction(ECluster_Dll, "ECluster_Slave_LogEvents")
    ECluster_Slave_Event = GetFunction(ECluster_Dll, "ECluster_Slave_Event")
    ECluster_Slave_Event_Type = GetFunction(ECluster_Dll, "ECluster_Slave_Event_Type")
    ECluster_Slave_Event_Master = GetFunction(ECluster_Dll, "ECluster_Slave_Event_Master")
    _ECluster_Slave_Event_Job = GetFunction(ECluster_Dll, "ECluster_Slave_Event_Job")
    ECluster_Slave_Job_SendMessage = GetFunction(ECluster_Dll, "ECluster_Slave_Job_SendMessage")
    
    ECluster_Master_Init = GetFunction(ECluster_Dll, "ECluster_Master_Init")
    ECluster_Master_DeInit = GetFunction(ECluster_Dll, "ECluster_Master_DeInit")
    _ECluster_Master_GetLastError = GetFunction(ECluster_Dll, "ECluster_Master_GetLastError")
    ECluster_Master_FreeStream = GetFunction(ECluster_Dll, "ECluster_Master_FreeStream")
    ECluster_Master_SetReceiveBlockSize = GetFunction(ECluster_Dll, "ECluster_Master_SetReceiveBlockSize")
    ECluster_Master_SetCompressionOptions = GetFunction(ECluster_Dll, "ECluster_Master_SetCompressionOptions")
    ECluster_Master_SetStreamOptions = GetFunction(ECluster_Dll, "ECluster_Master_SetStreamOptions")
    ECluster_Master_ThreadReceive = GetFunction(ECluster_Dll, "ECluster_Master_ThreadReceive")
    ECluster_Master_SetCallback = GetFunction(ECluster_Dll, "ECluster_Master_SetCallback")
    ECluster_Master_AddSlave = GetFunction(ECluster_Dll, "ECluster_Master_AddSlave")
    ECluster_Master_RemoveSlave = GetFunction(ECluster_Dll, "ECluster_Master_RemoveSlave")
    ECluster_Master_CountSlaves = GetFunction(ECluster_Dll, "ECluster_Master_CountSlaves")
    ECluster_Master_IterateSlaves = GetFunction(ECluster_Dll, "ECluster_Master_IterateSlaves")
    ECluster_Master_GetSlaveIP = GetFunction(ECluster_Dll, "ECluster_Master_GetSlaveIP")
    ECluster_Master_GetSlaveXtraData = GetFunction(ECluster_Dll, "ECluster_Master_GetSlaveXtraData")
    ECluster_Master_SetSlaveXtraData = GetFunction(ECluster_Dll, "ECluster_Master_SetSlaveXtraData")
    ECluster_Master_PingSlave = GetFunction(ECluster_Dll, "ECluster_Master_PingSlave")
    _ECluster_Master_GetSlaveJobInfos = GetFunction(ECluster_Dll, "ECluster_Master_GetSlaveJobInfos")
    ECluster_Master_CountSlaveJobs = GetFunction(ECluster_Dll, "ECluster_Master_CountSlaveJobs")
    ECluster_Master_CountSlaveWorkers = GetFunction(ECluster_Dll, "ECluster_Master_CountSlaveWorkers")
    ECluster_Master_DoJob = GetFunction(ECluster_Dll, "ECluster_Master_DoJob")
    
    ECluster_Master_WaitEvent = GetFunction(ECluster_Dll, "ECluster_Master_WaitEvent")
    ECluster_Master_Event = GetFunction(ECluster_Dll, "ECluster_Master_Event")
    ECluster_Master_Event_Type = GetFunction(ECluster_Dll, "ECluster_Master_Event_Type")
    ECluster_Master_Event_Slave = GetFunction(ECluster_Dll, "ECluster_Master_Event_Slave")
    ECluster_Master_Event_JobId = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobId")
    ECluster_Master_Event_JobMem = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobMem")
    ECluster_Master_Event_JobMemSize = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobMemSize")
    ECluster_Master_Event_JobLong = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobLong")
    ECluster_Master_Event_JobInteger = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobInteger")
    ECluster_Master_Event_JobFloat = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobFloat")
    ECluster_Master_Event_JobDouble = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobDouble")
    ECluster_Master_Event_JobQuad = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobQuad")
    _ECluster_Master_Event_JobString = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobString")
    
    ECluster_Cache_New = GetFunction(ECluster_Dll, "ECluster_Cache_New")
    ECluster_Cache_Config = GetFunction(ECluster_Dll, "ECluster_Cache_Config")
    ECluster_Cache_CleanUp = GetFunction(ECluster_Dll, "ECluster_Cache_CleanUp")
    ECluster_Cache_Optimize = GetFunction(ECluster_Dll, "ECluster_Cache_Optimize")
    ECluster_Cache_Flush = GetFunction(ECluster_Dll, "ECluster_Cache_Flush")
    ECluster_Cache_Free = GetFunction(ECluster_Dll, "ECluster_Cache_Free")
    ECluster_Cache_SaveToFile = GetFunction(ECluster_Dll, "ECluster_Cache_SaveToFile")
    ECluster_Cache_LoadFromFile = GetFunction(ECluster_Dll, "ECluster_Cache_LoadFromFile")
    ECluster_Cache_GetSize = GetFunction(ECluster_Dll, "ECluster_Cache_GetSize")
    ECluster_Cache_isCached = GetFunction(ECluster_Dll, "ECluster_Cache_isCached")
    ECluster_Cache_AddEntry = GetFunction(ECluster_Dll, "ECluster_Cache_AddEntry")
    ECluster_Cache_DeleteEntry = GetFunction(ECluster_Dll, "ECluster_Cache_DeleteEntry")
    ECluster_Cache_Examine = GetFunction(ECluster_Dll, "ECluster_Cache_Examine")
    ECluster_Cache_NextEntry = GetFunction(ECluster_Dll, "ECluster_Cache_NextEntry")
    ECluster_Cache_GetMem = GetFunction(ECluster_Dll, "ECluster_Cache_GetMem")
    ECluster_Cache_GetMemSize = GetFunction(ECluster_Dll, "ECluster_Cache_GetMemSize")
    ECluster_Cache_GetAge = GetFunction(ECluster_Dll, "ECluster_Cache_GetAge")
    ECluster_Cache_GetXtraData = GetFunction(ECluster_Dll, "ECluster_Cache_GetXtraData")
    _ECluster_Cache_GetName = GetFunction(ECluster_Dll, "ECluster_Cache_GetName")
    
    ECluster_SQLite_Create = GetFunction(ECluster_Dll, "ECluster_SQLite_Create")
    ECluster_SQLite_Open = GetFunction(ECluster_Dll, "ECluster_SQLite_Open")
    ECluster_SQLite_Close = GetFunction(ECluster_Dll, "ECluster_SQLite_Close")
    ECluster_SQLite_Update = GetFunction(ECluster_Dll, "ECluster_SQLite_Update")
    ECluster_SQLite_UpdateSync = GetFunction(ECluster_Dll, "ECluster_SQLite_UpdateSync")
    ECluster_SQLite_Query = GetFunction(ECluster_Dll, "ECluster_SQLite_Query")
    ECluster_SQLite_EndQuery = GetFunction(ECluster_Dll, "ECluster_SQLite_EndQuery")
    ECluster_SQLite_SelectRow = GetFunction(ECluster_Dll, "ECluster_SQLite_SelectRow")
    _ECluster_SQLite_GetAsString = GetFunction(ECluster_Dll, "ECluster_SQLite_GetAsString")
    ECluster_SQLite_GetAsLong = GetFunction(ECluster_Dll, "ECluster_SQLite_GetAsLong")
    ECluster_SQLite_GetAsQuad = GetFunction(ECluster_Dll, "ECluster_SQLite_GetAsQuad")
    ECluster_SQLite_GetAsDouble = GetFunction(ECluster_Dll, "ECluster_SQLite_GetAsDouble")
    ECluster_SQLite_GetAsBlobPointer = GetFunction(ECluster_Dll, "ECluster_SQLite_GetAsBlobPointer")
    ECluster_SQLite_GetAsBlobSize = GetFunction(ECluster_Dll, "ECluster_SQLite_GetAsBlobSize")
    ECluster_SQLite_GetAsBlobFlags = GetFunction(ECluster_Dll, "ECluster_SQLite_GetAsBlobFlags")
    ECluster_SQLite_GetNumColumns = GetFunction(ECluster_Dll, "ECluster_SQLite_GetNumColumns")
    _ECluster_SQLite_GetColumnTypeAsString = GetFunction(ECluster_Dll, "ECluster_SQLite_GetColumnTypeAsString")
    _ECluster_SQLite_GetColumnName = GetFunction(ECluster_Dll, "ECluster_SQLite_GetColumnName")
    ECluster_SQLite_StoreRow = GetFunction(ECluster_Dll, "ECluster_SQLite_StoreRow")
    ECluster_SQLite_StoreAsLong = GetFunction(ECluster_Dll, "ECluster_SQLite_StoreAsLong")
    ECluster_SQLite_StoreAsQuad = GetFunction(ECluster_Dll, "ECluster_SQLite_StoreAsQuad")
    ECluster_SQLite_StoreAsDouble = GetFunction(ECluster_Dll, "ECluster_SQLite_StoreAsDouble")
    ECluster_SQLite_StoreAsString = GetFunction(ECluster_Dll, "ECluster_SQLite_StoreAsString")
    ECluster_SQLite_StoreAsBlob = GetFunction(ECluster_Dll, "ECluster_SQLite_StoreAsBlob")
    _ECluster_SQLite_GetLastErrorString = GetFunction(ECluster_Dll, "ECluster_SQLite_GetLastErrorString")
    ECluster_SQLite_SetCompressionLevel = GetFunction(ECluster_Dll, "ECluster_SQLite_SetCompressionLevel")
    ECluster_SQLite_Flush = GetFunction(ECluster_Dll, "ECluster_SQLite_Flush")
    _ECluster_SQLite_GetSQLiteVersion = GetFunction(ECluster_Dll, "ECluster_SQLite_GetSQLiteVersion")
    
    ECluster_Parameter_New = GetFunction(ECluster_Dll, "ECluster_Parameter_New")
    ECluster_Parameter_AddMem = GetFunction(ECluster_Dll, "ECluster_Parameter_AddMem")
    ECluster_Parameter_AddLong = GetFunction(ECluster_Dll, "ECluster_Parameter_AddLong")
    ECluster_Parameter_AddInteger = GetFunction(ECluster_Dll, "ECluster_Parameter_AddInteger")
    ECluster_Parameter_AddFloat = GetFunction(ECluster_Dll, "ECluster_Parameter_AddFloat")
    ECluster_Parameter_AddDouble = GetFunction(ECluster_Dll, "ECluster_Parameter_AddDouble")
    ECluster_Parameter_AddQuad = GetFunction(ECluster_Dll, "ECluster_Parameter_AddQuad")
    ECluster_Parameter_AddString = GetFunction(ECluster_Dll, "ECluster_Parameter_AddString")
    ECluster_Parameter_GetMem = GetFunction(ECluster_Dll, "ECluster_Parameter_GetMem")
    ECluster_Parameter_GetLong = GetFunction(ECluster_Dll, "ECluster_Parameter_GetLong")
    ECluster_Parameter_GetInteger = GetFunction(ECluster_Dll, "ECluster_Parameter_GetInteger")
    ECluster_Parameter_GetFloat = GetFunction(ECluster_Dll, "ECluster_Parameter_GetFloat")
    ECluster_Parameter_GetDouble = GetFunction(ECluster_Dll, "ECluster_Parameter_GetDouble")
    ECluster_Parameter_GetQuad = GetFunction(ECluster_Dll, "ECluster_Parameter_GetQuad")
    _ECluster_Parameter_GetString = GetFunction(ECluster_Dll, "ECluster_Parameter_GetString")
    
    ECluster_Return_Memory = GetFunction(ECluster_Dll, "ECluster_Return_Memory")
    ECluster_Return_Long = GetFunction(ECluster_Dll, "ECluster_Return_Long")
    ECluster_Return_Integer = GetFunction(ECluster_Dll, "ECluster_Return_Integer")
    ECluster_Return_Float = GetFunction(ECluster_Dll, "ECluster_Return_Float")
    ECluster_Return_Double = GetFunction(ECluster_Dll, "ECluster_Return_Double")
    ECluster_Return_Quad = GetFunction(ECluster_Dll, "ECluster_Return_Quad")
    ECluster_Return_String = GetFunction(ECluster_Dll, "ECluster_Return_String")
    
    ECluster_AllocateMemory = GetFunction(ECluster_Dll, "ECluster_AllocateMemory")
    ECluster_FreeMemory = GetFunction(ECluster_Dll, "ECluster_FreeMemory")
    
    _ECluster_GetVersion = GetFunction(ECluster_Dll, "ECluster_GetVersion")
    
    ProcedureReturn ECluster_Dll
  EndIf
EndProcedure

; Automatisches Initialisieren
Macro ECluster_AutoInit()
  CompilerSelect #PB_Compiler_OS
    CompilerCase #PB_OS_Windows
      CompilerIf #PB_Compiler_Unicode
        ECluster_Init(#ECluster_IncludePath + "Windows\ECluster_Unicode.dll")
      CompilerElse
        ECluster_Init(#ECluster_IncludePath + "Windows\ECluster.dll")
      CompilerEndIf
    CompilerCase #PB_OS_Linux
      CompilerIf #PB_Compiler_Unicode
        ECluster_Init(#ECluster_IncludePath + "Linux/ECluster_Unicode.so")
      CompilerElse
        ECluster_Init(#ECluster_IncludePath + "Linux/ECluster.so")
      CompilerEndIf
  CompilerEndSelect
EndMacro

; IDE Options = PureBasic 4.31 (Windows - x86)
; CursorPosition = 67
; FirstLine = 50
; Folding = -
; EnableThread
; EnableXP
; DisableDebugger