; ------------------------------------------------------------------------------------
; ECluster Cache Funktionen
;
; Stellt einfache Funktionen fr einen Anwendungscache bereit, sehr ntzlich fr
; Serveranwendungen.
;
; ------------------------------------------------------------------------------------


; ------------------------------------------------------------------------------------
;- Cache Routinen
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; Legt einen neuen Cache an.
;
; MaximumSize: Maximalgre in Bytes
; AgeTolerance: Alterstolleranz fr autmatische Bereinigung in Sekunden
; AccessTolerance: Zugriffstolleranz fr autmatische Bereinigung
;
; Rckgabe: Cache - Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_New(MaximumSize = #Null, AgeTolerance = #Null, AccessTolerance = #Null)

; ------------------------------------------------------------------------------------
; Konfiguriert einen Cache
;
; ECluster_Cache: Cache - Handle
; MaximumSize: Maximalgre in Bytes (0 = Unbegrenzt)
; AgeTolerance: Alterstolleranz fr autmatische Bereinigung in Sekunden
; AccessTolerance: Zugriffstolleranz fr autmatische Bereinigung
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_Config(ECluster_Cache, MaximumSize = #Null, AgeTolerance = #Null, AccessTolerance = #Null)

; ------------------------------------------------------------------------------------
; Bereinigt den Cache manuell
;
; ECluster_Cache: Cache - Handle
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_CleanUp(ECluster_Cache)

; ------------------------------------------------------------------------------------
; Optimiert den Cache
;
; ECluster_Cache: Cache - Handle
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_Optimize(ECluster_Cache)

; ------------------------------------------------------------------------------------
; Lscht den gesamten Inhalt des Cache
;
; ECluster_Cache: Cache - Handle
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_Flush(ECluster_Cache)

; ------------------------------------------------------------------------------------
; Gibt den Cache frei
;
; ECluster_Cache: Cache - Handle
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_Free(ECluster_Cache)

; ------------------------------------------------------------------------------------
; Speicher den Cache in eine Datei
;
; ECluster_Cache: Cache - Handle
; FileName$: Dateiname
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_SaveToFile(ECluster_Cache, FileName$)

; ------------------------------------------------------------------------------------
; Ld den Cache aus einer Datei die mit ECluster_Cache_SaveToFile() gespeichert wurde.
;
; ECluster_Cache: Cache - Handle
; FileName$: Dateiname
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_LoadFromFile(ECluster_Cache, FileName$)

; ------------------------------------------------------------------------------------
; Aktuelle Gre des Cache
;
; ECluster_Cache: Cache - Handle
;
; Rckgabe: Cachegre
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_GetSize(ECluster_Cache)

; ------------------------------------------------------------------------------------
; Prft ob ein Eintrag im Cache vorhanden ist
;
; ECluster_Cache: Cache - Handle
; Name$: Eintragsname
;
; Rckgabe: #True falls vorhanden
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_isCached(ECluster_Cache, Name$)

; ------------------------------------------------------------------------------------
; Speichert einen Eintrag im Cache
;
; ECluster_Cache: Cache - Handle
; Name$: Eintragsname
; *Mem: Speicheraddresse
; MemSize: Speichergre
; MaxAge: Maximalalter der Daten in Sekunden (0 = Unbegrenzt)
; XtraData: Xtrainformationen
;
; Rckgabe: #True falls vorhanden
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_AddEntry(ECluster_Cache, Name$, *Mem, MemSize, MaxAge = #Null, XtraData = #Null)

; ------------------------------------------------------------------------------------
; Lscht einen Eintrag manuell aus dem Cache
;
; ECluster_Cache: Cache - Handle
; Name$: Eintragsname
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_DeleteEntry(ECluster_Cache, Name$)

; ------------------------------------------------------------------------------------
; Beginnt eine Iteration aller Daten im Cache
;
; ECluster_Cache: Cache - Handle
;
; R+ckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_Examine(ECluster_Cache)

; ------------------------------------------------------------------------------------
; Holt den nchsten Cache - Eintrag (Nach ECluster_Cache_Examine()
;
; ECluster_Cache: Cache - Handle
;
; R+ckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_NextEntry(ECluster_Cache)

; ------------------------------------------------------------------------------------
; Holt die Speicheraddresse des aktullen Cache-Eintrags
; Zu verwenden nach ECluster_Cache_NextEntry() oder ECluster_Cache_isCached()
;
; ECluster_Cache: Cache - Handle
;
; Rckgabe: Speicheraddresse
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_GetMem(ECluster_Cache)

; ------------------------------------------------------------------------------------
; Holt die Speichergre des aktullen Cache-Eintrags
; Zu verwenden nach ECluster_Cache_NextEntry() oder ECluster_Cache_isCached()
;
; ECluster_Cache: Cache - Handle
;
; Rckgabe: Speichergre
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_GetMemSize(ECluster_Cache)

; ------------------------------------------------------------------------------------
; Holt das Alter in Sekunden des aktullen Cache-Eintrags
; Zu verwenden nach ECluster_Cache_NextEntry() oder ECluster_Cache_isCached()
;
; ECluster_Cache: Cache - Handle
;
; Rckgabe: Alter
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_GetAge(ECluster_Cache)

; ------------------------------------------------------------------------------------
; Holt die XtraDaten des aktullen Cache-Eintrags
; Zu verwenden nach ECluster_Cache_NextEntry() oder ECluster_Cache_isCached()
;
; ECluster_Cache: Cache - Handle
;
; Rckgabe: XtraDaten
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_GetXtraData(ECluster_Cache)

; ------------------------------------------------------------------------------------
; Holt den Namen des aktullen Cache-Eintrags
; Zu verwenden nach ECluster_Cache_NextEntry() oder ECluster_Cache_isCached()
;
; ECluster_Cache: Cache - Handle
;
; Rckgabe: Eintragsname
; ------------------------------------------------------------------------------------

Prototype _ECluster_Cache_GetName(ECluster_Cache)


; ------------------------------------------------------------------------------------
;- Funktionen Deklarieren
; ------------------------------------------------------------------------------------

Global ECluster_Cache_New._ECluster_Cache_New
Global ECluster_Cache_Config._ECluster_Cache_Config
Global ECluster_Cache_CleanUp._ECluster_Cache_CleanUp
Global ECluster_Cache_Optimize._ECluster_Cache_Optimize
Global ECluster_Cache_Flush._ECluster_Cache_Flush
Global ECluster_Cache_Free._ECluster_Cache_Free
Global ECluster_Cache_SaveToFile._ECluster_Cache_SaveToFile
Global ECluster_Cache_LoadFromFile._ECluster_Cache_LoadFromFile
Global ECluster_Cache_GetSize._ECluster_Cache_GetSize
Global ECluster_Cache_isCached._ECluster_Cache_isCached
Global ECluster_Cache_AddEntry._ECluster_Cache_AddEntry
Global ECluster_Cache_DeleteEntry._ECluster_Cache_DeleteEntry
Global ECluster_Cache_Examine._ECluster_Cache_Examine
Global ECluster_Cache_NextEntry._ECluster_Cache_NextEntry
Global ECluster_Cache_GetMem._ECluster_Cache_GetMem
Global ECluster_Cache_GetMemSize._ECluster_Cache_GetMemSize
Global ECluster_Cache_GetAge._ECluster_Cache_GetAge
Global ECluster_Cache_GetXtraData._ECluster_Cache_GetXtraData
Global _ECluster_Cache_GetName._ECluster_Cache_GetName

Macro ECluster_Cache_GetName(_Cache)
  PeekS(_ECluster_Cache_GetName(_Cache))
EndMacro

; IDE Options = PureBasic 4.31 (Windows - x86)
; CursorPosition = 8
; Folding = -
; EnableXP