; ------------------------------------------------------------------------------------
; Master Routinen
; Clientteil von ECluster
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
;- Strukturen
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; Stream - Objekt
; ------------------------------------------------------------------------------------

Structure ECluster_Master_StreamObject ; Stream - Objekt
  *StreamMem ; StreamMemory
  StreamMemSize.l ; Gesamte StreamMemory Gre
  CurrentStreamMemSize.l ; Aktuelle StreamMemoryGre
  StreamStatus.l ; StreamStatus
  StreamCancel.l ; Stream abbrechen
EndStructure

; ------------------------------------------------------------------------------------
;- Enumerationen
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; Stream Statusmglichkeiten
; ------------------------------------------------------------------------------------

Enumeration 1
  #Stream_InProgress = $01 ; Stream beim bertragen
  #Stream_Finished = $02; Stream Fertig
  #Stream_Error = $03; Fehler, Stream abgebrochen
  #Stream_Canceled = $04 ; Stream abgebrochen
EndEnumeration

; ------------------------------------------------------------------------------------
; Mgliche Rckgabewerte von ECluster_Master_Event_Type()
; ------------------------------------------------------------------------------------

Enumeration 1
  ; Job Events (01 - 9F)
  #Event_Job_Finished = $01 ; Job beendet
  #Event_Job_InProgress = $02 ; Job in Arbeit
  #Event_Job_Received = $03 ; Job wurde empfangen
  #Event_Job_Message = $04 ; Meldung vom Job
  #Event_Job_Limit = $05 ; Job Limit erreicht, wird nicht ausgefhrt
  #Event_Job_Unknown = $06 ; Job unbekannt
  #Event_Job_Denied = $07 ; Job wurde verweigert
  #Event_Job_Invalid = $08 ; Ungltige Job - Id
  #Event_Job_Error = $9F ; Job Fehler (wegen Buffer-berlauf)
  
  ; Slave Events (A0 - EF)
  #Event_Slave_Quit = $A0 ; Slave beendet
  #Event_Slave_Paused = $A1 ; Slave Pausiert
  #Event_Slave_Resumed = $A2 ; Slave macht weiter
  #Event_Slave_Kick = $A3; Master wurde gekickt
  #Event_Slave_UnknownCommand = $A4; Unbekanntes Kommando
  
  ; Sonstiges (F0 - FF)
  #Event_Master_BufferUnderrun = $FE; Buffer Unterlauf
  #Event_Slave_BufferUnderrun = $FF; Buffer Unterlauf
EndEnumeration

; ------------------------------------------------------------------------------------
; Proxy-Typen fr ECluster_Master_AddSlave()
; ------------------------------------------------------------------------------------

Enumeration 0
  #Proxy_HTTP = 0 ; HTTP - Proxy
  #Proxy_SOCKS4 = 0 ; SOCKS4 - Proxy
  #Proxy_SOCKS5 = 0 ; SOCKS5 - Proxy
EndEnumeration

; ------------------------------------------------------------------------------------
; Priorittsklassen fr Jobs
; ------------------------------------------------------------------------------------

Enumeration 0 ; Priority - Klassen
  #Job_LowPriority
  #Job_NormalPriority
  #Job_HighPriority
  #Job_HighestPriority
  #Job_RealTimePriority
EndEnumeration

; ------------------------------------------------------------------------------------
;- Callbacks
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; Callback fr ECluster_Master_IterateSlaves()
; ECluster_Master: Master Handle
; SlaveNumber: aktuelle Slave Nummer
; UserData: angegebene UserData
;
; Rckgabe: <> #False: Iteration fortsetzen, #False: Iteration abbrecheb
; ------------------------------------------------------------------------------------

Prototype ECluster_Master_SlaveCB(ECluster_Master, SlaveNumber, UserData)

; ------------------------------------------------------------------------------------
; Callback fr Master - Events
;
; ECluster_Master: Master Handle
; SlaveNumber: aktuelle Slave Nummer
; Job: Job Nummer
; EventType: EventTyp (wie ECluster_Master_Event_Type())
; *Mem: Speicheradresse (wenn EventType = #Event_Job_Finished)
; MemSize: Speichergre (wenn EventType = #Event_Job_Finished)
; UserData: angegebene UserData
;
; Rckgabe: <> #False: Iteration fortsetzen, #False: Iteration abbrecheb
; ------------------------------------------------------------------------------------

Prototype ECluster_Master_EventCB(ECluster_Master, SlaveNumber, Job, EventType, *Mem, MemSize, UserData)

; ------------------------------------------------------------------------------------
; Callback fr ausgehende Streams
;
; ECluster_Master: Master Handle
; SlaveNumber: Slave - Nummer
; GesMem: Gesamte Speichergre
; SendMem: Gesendete Speichergre
; UserData: UserDaten
;
; Rckgabe: <> 0: Fortsetzen, =0 Stream abbrechen
; ------------------------------------------------------------------------------------

Prototype ECluster_Master_StreamCB(ECluster_Master, SlaveNumber, GesMem, SendMem, UserData)

; ------------------------------------------------------------------------------------
; Callback fr eingehende Streams
;
; ECluster_Master: Master Handle
; SlaveNumber: Slave - Nummer
; StreamMaster: Von welchen Master kommt der Stream
; StreamName: Name des Stream
; StreamParas: Parameter des Stream
; *Stream: StreamObjekt
; ------------------------------------------------------------------------------------

Prototype ECluster_Master_IncomingStream(ECluster_Master, SlaveNumber, StreamMaster, StreamName$, StreamParas, *Stream.ECluster_Master_StreamObject)

; ------------------------------------------------------------------------------------
;- Master Routinen
;
; Routinen zur Steuerung eines Master
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; Initialisiert einen neuen Master
;
; *Stream: CB fr eigehende Streams (Bentigt)
;
; Rckgabe: Master - Handle wenn Ok, #False wenn Fehler
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_Init(*Stream.ECluster_Master_IncomingStream)

; ------------------------------------------------------------------------------------
; Beendet einen Master
;
; ECluster_Master: Master Handle
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_DeInit(ECluster_Master)

; ------------------------------------------------------------------------------------
; Beschreibung des letzen Fehlers
;
; ECluster_Master: Master Handle
;
; Rckgabe: Fehlermeldung
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_GetLastError(ECluster_Master)

; ------------------------------------------------------------------------------------
; Gibt ein Stream-Objekt frei
;
; ECluster_Master: Master Handle
; *Stream: Stream Objekt
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_FreeStream(ECluster_Master, *Stream)

; ------------------------------------------------------------------------------------
; Setzt die EmpfangsBlock - Puffergre
;
; ECluster_Master: Slave Handle
; Size: neue EmpfangsBlock - Puffergre in Bytes > 0 (Default: 512 Kb)
;
; Rckgabe: Alte Puffergre
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_SetReceiveBlockSize(ECluster_Master, Size = 512*1024)

; ------------------------------------------------------------------------------------
; Setzt die Einstellungen fr die Automatische Kompression
;
; ECluster_Master: Master Handle
; Limit: Limit ab dem Komprimiert werden soll (0: deaktiviert, <0: alles komrimieren)
; Level: Kompressions-Level (1-9, 0 = deaktiviert)
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_SetCompressionOptions(ECluster_Master, Limit, Level)

; ------------------------------------------------------------------------------------
; Setzt die Einstellungen fr Daten Streams
;
; ECluster_Master: Master Handle
; BlockOrderTimeout: Wie lange soll auf eine Anfrage fr den nchsten Block vom SLave gewartet werden? (in Ms, Default 5000)
; MaximumResends: Wie oft soll ein fehlgeschlagener Block neugesendet werden? (Default 3)
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_SetStreamOptions(ECluster_Master, BlockOrderTimeout, MaximumResends)

; ------------------------------------------------------------------------------------
; Setzt die Einstellungen fr ein globales Callback
;
; ECluster_Master: Master Handle
; *CallBack: CallBack (siehe ECluster_Master_EventCB)
; UserData: UserData fr Callback
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_SetCallback(ECluster_Master, *CallBack, UserData)

; ------------------------------------------------------------------------------------
; Aktiviert den Thread Empfangsmodus des Slave
;
; ECluster_Master: Master Handle
; State: <> 0: Aktiviert, 0: Deaktiviert
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_ThreadReceive(ECluster_Master, State)

; ------------------------------------------------------------------------------------
; Fgt einen Slave zum Master hinzu
;
; ECluster_Master: Master Handle
; Address$: Netzwerkaddresse des Slave
; Port: Portnummer des Slave
; ProxyAddress: Netzwerkaddresse des Proxy falls Verbindung ber Proxy
; ProxyPort: Portnummer des Proxy
; ProxyUser: Username des Proxy (Nur fr SOCKS5 - Proxy)
; ProxyPass: Passwort fr den Proxy (Nur fr SOCKS5 - Proxy)
; ProxyType: Typ des Proxy (HTTP/SOCKS4/SOCKS5)
; ProxyTimeout: Verbindungstimeout zum Proxy in Sekunden

;
; Rckgabe: Slave - Nummer wenn Ok, #False wenn Fehler
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_AddSlave(ECluster_Master, Address$, Port, ProxyAddress.s = "", ProxyPort = 0, ProxyUser.s = "", ProxyPass.s = "", ProxyType = #Proxy_HTTP, ProxyTimeout = 5)

; ------------------------------------------------------------------------------------
; Entfernt einen Slave vom Master
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_RemoveSlave(ECluster_Master, Slave)

; ------------------------------------------------------------------------------------
; Zhlt die Anzahl der Slaves am Master
;
; ECluster_Master: Master Handle
;
; Rckgabe: Slave Anzahl
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_CountSlaves(ECluster_Master)

; ------------------------------------------------------------------------------------
; Iteriert die Slaves des Masters und sendet sie an ein CallBack
;
; ECluster_Master: Master Handle
; *CallBack: Pointer auf ECluster_Master_SlaveCB Procedure
; UserData: Individuelle Daten
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_IterateSlaves(ECluster_Master, *CallBack.ECluster_Master_SlaveCB, UserData)

; ------------------------------------------------------------------------------------
; Ermittelt die IP des Slave (zur Verarbeitung mit IPString())
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
;
; Rckgabe: SlaveIP
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_GetSlaveIP(ECluster_Master, Slave)

; ------------------------------------------------------------------------------------
; Holt die gespeicherten Xtra-Daten des Slave
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
;
; Rckgabe: Xtra - Data des Slave
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_GetSlaveXtraData(ECluster_Master, Slave)

; ------------------------------------------------------------------------------------
; Setzt die Xtra-Daten des Slave
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
; XtraData: XtraDaten
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_SetSlaveXtraData(ECluster_Master, Slave, XtraData)

; ------------------------------------------------------------------------------------
; Pingt den Slave
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
;
; Rckgabe: Ping Zeit in Ms, -1 bei Fehler
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_PingSlave(ECluster_Master, Slave)

; ------------------------------------------------------------------------------------
; Frag eine Job Liste vom Slave an
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
; ForceRefresh: Liste neu anfordern wenn schon geladen
;
; Rckgabe: Job Liste incl. Beschreibungen
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_GetSlaveJobInfos(ECluster_Master, Slave, ForceRefresh = 0)

; ------------------------------------------------------------------------------------
; Frag die Anzahl von anstehenden Jobs am Slave ab
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
;
; Rckgabe: Anzahl Jobs im JobQueue des Slave, -1 bei Fehler
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_CountSlaveJobs(ECluster_Master, Slave)

; ------------------------------------------------------------------------------------
; Frag die Anzahl der Worker am Slave ab
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
;
; Rckgabe: Anzahl Jobs im JobQueue des Slave, -1 bei Fehler
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_CountSlaveWorkers(ECluster_Master, Slave)

; ------------------------------------------------------------------------------------
; Fhrt einen Job am Slave aus
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
; ParameterStack: ParameterStack (ECluster_Parameter_New())
; *CallBack: CallBack fr Optinales EventCB (siehe ECluster_Master_EventCB)
; UserData: UserData fr EventCB
; *StreamMem: Speicher fr einen MemoryStream (Optional)
; StreamMemSize: Speichergre des StreamSpeichers
; StreamBlockSize: Blockgre fr Speicherblcke, falls 0 256 Kb (StreamMemSize muss NICHT genau durch StreamBlockSize teilbar sein)
; *StreamCallback: Pointer auf ein Callback fr die Fortschrittsanzeige des Stream (siehe ECluster_Master_StreamCB())
; StreamCBUserData: Userdaten fr das Stream-Callback
; StreamTarget: Stream-Ziel. (<>0 = Master, P2P Stream!)
; Priority: Job Prioritt
;
; Rckgabe: JobId wenn Ok, #False wenn Fehler
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_DoJob(ECluster_Master, JobName$, Slave, ParameterStack, *CallBack = 0, UserData = 0, *StremMem = 0, StremMemSize = 0, StreamBlockSize = 0, *StreamCallback = 0, StreamCBUserData = 0, StreamTarget = 0, Priority = #Job_NormalPriority)

; ------------------------------------------------------------------------------------
; Wartet bis ein Event eintritt
;
; ECluster_Master: Master Handle
; TimeOut: Optinales Timeout
;
; Rckgabe: 0>: JobId , #Null: Fehler beim Senden
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_WaitEvent(ECluster_Master, TimeOut = 0)

; ------------------------------------------------------------------------------------
; Prft ob Events vorhanden sind
;
; ECluster_Master: Master Handle
;
; Rckgabe: Event-Anzahl wenn vorhanden, sonst #False
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_Event(ECluster_Master)

; ------------------------------------------------------------------------------------
; Typ des aktuellen Events
;
; ECluster_Master: Master Handle
;
; Rckgabe: EventType
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_Event_Type(ECluster_Master)

; ------------------------------------------------------------------------------------
; Slave des aktuellen Events
;
; ECluster_Master: Master Handle
;
; Rckgabe: EventSlave
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_Event_Slave(ECluster_Master)

; ------------------------------------------------------------------------------------
; JobId des aktuellen Events
;
; ECluster_Master: Master Handle
;
; Rckgabe: JobId
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_Event_JobId(ECluster_Master)

; ------------------------------------------------------------------------------------
; Folgende Routinen holen den Rckgabewert eines Jobs
; ACHTUNG: Jeder Wert kann nur 1* gelesen werden
;
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; RckgabeSpeicher des aktuellen EventJobs
; ACHTUNG: Wenn sie den Speicher mit diesem Befehl selbst holen mssen sie den Speicher
; und die Structur selbst freigeben.
;
; ECluster_Master: Master Handle
;
; Rckgabe: Pointer auf eine ECluster_Mem Structur
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_Event_JobMem(ECluster_Master)

; ------------------------------------------------------------------------------------
; RckgabeSpeicher - Gre des aktuellen EventJobs
;
; ECluster_Master: Master Handle
;
; Rckgabe: Speichergre
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_Event_JobMemSize(ECluster_Master)

; ------------------------------------------------------------------------------------
; RckgabeSpeicher des aktuellen EventJobs als Long
;
; ECluster_Master: Master Handle
;
; Rckgabe: JobResult
; ------------------------------------------------------------------------------------

Prototype.l _ECluster_Master_Event_JobLong(ECluster_Master)

; ------------------------------------------------------------------------------------
; RckgabeSpeicher des aktuellen EventJobs als Integer
;
; ECluster_Master: Master Handle
;
; Rckgabe: JobResult
; ------------------------------------------------------------------------------------

Prototype.i _ECluster_Master_Event_JobInteger(ECluster_Master)

; ------------------------------------------------------------------------------------
; RckgabeSpeicher des aktuellen EventJobs als Float
;
; ECluster_Master: Master Handle
;
; Rckgabe: JobResult
; ------------------------------------------------------------------------------------

Prototype.f _ECluster_Master_Event_JobFloat(ECluster_Master)

; ------------------------------------------------------------------------------------
; RckgabeSpeicher des aktuellen EventJobs als Double
;
; ECluster_Master: Master Handle
;
; Rckgabe: JobResult
; ------------------------------------------------------------------------------------

Prototype.d _ECluster_Master_Event_JobDouble(ECluster_Master)

; ------------------------------------------------------------------------------------
; RckgabeSpeicher des aktuellen EventJobs als Quad
;
; ECluster_Master: Master Handle
;
; Rckgabe: JobResult
; ------------------------------------------------------------------------------------

Prototype.q _ECluster_Master_Event_JobQuad(ECluster_Master)

; ------------------------------------------------------------------------------------
; RckgabeSpeicher des aktuellen EventJobs als String
;
; ECluster_Master: Master Handle
;
; Rckgabe: JobResult
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_Event_JobString(ECluster_Master)

; ------------------------------------------------------------------------------------
;- Funktionen Deklarieren
; ------------------------------------------------------------------------------------

Global ECluster_Master_Init._ECluster_Master_Init
Global ECluster_Master_DeInit._ECluster_Master_DeInit
Global _ECluster_Master_GetLastError._ECluster_Master_GetLastError
Global ECluster_Master_FreeStream._ECluster_Master_FreeStream
Global ECluster_Master_SetReceiveBlockSize._ECluster_Master_SetReceiveBlockSize
Global ECluster_Master_SetCompressionOptions._ECluster_Master_SetCompressionOptions
Global ECluster_Master_SetStreamOptions._ECluster_Master_SetStreamOptions
Global ECluster_Master_SetCallback._ECluster_Master_SetCallback
Global ECluster_Master_ThreadReceive._ECluster_Master_ThreadReceive
Global ECluster_Master_AddSlave._ECluster_Master_AddSlave
Global ECluster_Master_RemoveSlave._ECluster_Master_RemoveSlave
Global ECluster_Master_CountSlaves._ECluster_Master_CountSlaves
Global ECluster_Master_IterateSlaves._ECluster_Master_IterateSlaves
Global ECluster_Master_GetSlaveIP._ECluster_Master_GetSlaveIP
Global ECluster_Master_GetSlaveXtraData._ECluster_Master_GetSlaveXtraData
Global ECluster_Master_SetSlaveXtraData._ECluster_Master_SetSlaveXtraData
Global ECluster_Master_PingSlave._ECluster_Master_PingSlave
Global _ECluster_Master_GetSlaveJobInfos._ECluster_Master_GetSlaveJobInfos
Global ECluster_Master_CountSlaveJobs._ECluster_Master_CountSlaveJobs
Global ECluster_Master_CountSlaveWorkers._ECluster_Master_CountSlaveWorkers
Global ECluster_Master_DoJob._ECluster_Master_DoJob

Global ECluster_Master_WaitEvent._ECluster_Master_WaitEvent
Global ECluster_Master_Event._ECluster_Master_Event
Global ECluster_Master_Event_Type._ECluster_Master_Event_Type
Global ECluster_Master_Event_Slave._ECluster_Master_Event_Slave
Global ECluster_Master_Event_JobId._ECluster_Master_Event_JobId
Global ECluster_Master_Event_JobMem._ECluster_Master_Event_JobMem
Global ECluster_Master_Event_JobMemSize._ECluster_Master_Event_JobMemSize
Global ECluster_Master_Event_JobLong._ECluster_Master_Event_JobLong
Global ECluster_Master_Event_JobInteger._ECluster_Master_Event_JobInteger
Global ECluster_Master_Event_JobFloat._ECluster_Master_Event_JobFloat
Global ECluster_Master_Event_JobDouble._ECluster_Master_Event_JobDouble
Global ECluster_Master_Event_JobQuad._ECluster_Master_Event_JobQuad
Global _ECluster_Master_Event_JobString._ECluster_Master_Event_JobString

; Stream Makro
Macro ECluster_Master_SendStream(_Master, _Slave, _StreamTargetMaster, _StreamName, _StreamParameter, _StreamMem, _StreamMemSize, _StreamBlockSize, _StreamCB)
  ECluster_Master_DoJob(_Master, _StreamName, _Slave, _StreamParameter, #Null, #Null, _StreamMem, _StreamMemSize, _StreamBlockSize, _StreamCB, _StreamTargetMaster)
EndMacro

Procedure.s ECluster_Master_GetLastError(ECluster_Master)
  Protected *StringMem = _ECluster_Master_GetLastError(ECluster_Master)
  If *StringMem
    ProcedureReturn PeekS(*StringMem)
  EndIf
EndProcedure

Procedure.s ECluster_Master_GetSlaveJobInfos(ECluster_Master, Slave, ForceRefresh = 0)
  Protected String$, *StringMem = _ECluster_Master_GetSlaveJobInfos(ECluster_Master, Slave, ForceRefresh)
  If *StringMem
    String$ = PeekS(*StringMem)
    ECluster_FreeMemory(*StringMem)
    ProcedureReturn String$
  EndIf
EndProcedure

Procedure.s ECluster_Master_Event_JobString(ECluster_Master)
  Protected String$, *StringMem = _ECluster_Master_Event_JobString(ECluster_Master)
  If *StringMem
    String$ = PeekS(*StringMem)
    ECluster_FreeMemory(*StringMem)
    ProcedureReturn String$
  EndIf
EndProcedure


; IDE Options = PureBasic 4.31 (Windows - x86)
; CursorPosition = 579
; FirstLine = 563
; Folding = -
; EnableXP