; ------------------------------------------------------------------------------------
; ECluster SQLite Funktionen
; Credits an: RocketRider
; http://www.rocketrider.eu/Forum/viewtopic.php?p=917#917
;
; Einfach zu benutzende SQLite Befehle
;
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
;- Enumerationen
; ------------------------------------------------------------------------------------

Enumeration
  #ECluster_SQLite_BLOB_DEFAULT = #Null
  #ECluster_SQLite_BLOB_COMPRESSED = 1
EndEnumeration

; ------------------------------------------------------------------------------------
;- SQLite Routinen
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; Erstellt eine neue SQLite - Datenbank
;
; File: Dateiname der Datenbank
;
; Rckgabe: ECLuster_SQLite - Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_Create(File.s)

; ------------------------------------------------------------------------------------
; ffnet eine bestehende SQLite - Datenbank
;
; File: Dateiname der Datenbank
;
; Rckgabe: ECLuster_SQLite - Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_Open(File.s)

; ------------------------------------------------------------------------------------
; Schliet eine SQLite Datenbank
;
; ECluster_SQLite: Datenbank - Handle
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_Close(ECluster_SQLite)

; ------------------------------------------------------------------------------------
; Fhrt eine SQL Anweisung aus. Anders als ECluster_SQLite_UpdateSync() sind alle
; Anweisungen in einer Transaction, was viel schneller ist. Beendet wird die
; Transaktion bei einem Aufruf von ECluster_SQLite_UpdateSync(), ECluster_SQLite_Query(),
; ECluster_SQLite_Close() oder ECluster_SQLite_Flush().
;
; ECluster_SQLite: Datenbank - Handle
; Statement: SQL - Anweisung
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_Update(ECluster_SQLite, Statement.s)

; ------------------------------------------------------------------------------------
; Fhrt eine SQL Anweisung aus
;
; ECluster_SQLite: Datenbank - Handle
; Statement: SQL - Anweisung
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_UpdateSync(ECluster_SQLite, Statement.s)

; ------------------------------------------------------------------------------------
; Fhrt eine SQL Abfrage aus. ECluster_SQLite_EndQuery() muss zum Beenden der Abfrage
; aufgerufen werden
; Achtung: ECluster_SQLite_Store...() und ECluster_SQLite_GetAs...() Befehle knnen
; nicht in der selben Abfrage verwendet werden.
;
; ECluster_SQLite: Datenbank - Handle
; Statement: SQL - Anweisung
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_Query(ECluster_SQLite, Statement.s)

; ------------------------------------------------------------------------------------
; Beendet eine SQL - Abfrage
;
; ECluster_SQLite: Datenbank - Handle
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_EndQuery(ECluster_SQLite)

; ------------------------------------------------------------------------------------
; Whlt eine Reihe aus. Betigt fr die ECluster_SQL_Get...() Befehle.
; Die 1. Reihe beginnt mit 0.
; Achtung: Diese Funktion muss in einem ECluster_SQLite_Query()...ECluster_SQLite_EndQuery()
; Block ausgefhrt werden.
;
; ECluster_SQLite: Datenbank - Handle
; Row: Reihe
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_SelectRow(ECluster_SQLite, Row)

; ------------------------------------------------------------------------------------
; Gibt den Wert der Spalte Column als String zurck. (Spalten beginnen bei 0)
; Achtung: ECluster_SQLite_SelectRow() muss vor diesem Befehl ausgefhrt werden
;
; ECluster_SQLite: Datenbank - Handle
; Column: Spalte
;
; Rckgabe: Spalteninhalt
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_GetAsString(ECluster_SQLite, Column)

; ------------------------------------------------------------------------------------
; Gibt den Wert der Spalte Column als Long zurck. (Spalten beginnen bei 0)
; Achtung: ECluster_SQLite_SelectRow() muss vor diesem Befehl ausgefhrt werden
;
; ECluster_SQLite: Datenbank - Handle
; Column: Spalte
;
; Rckgabe: Spalteninhalt
; ------------------------------------------------------------------------------------

Prototype.l _ECluster_SQLite_GetAsLong(ECluster_SQLite, Column)

; ------------------------------------------------------------------------------------
; Gibt den Wert der Spalte Column als Quad zurck. (Spalten beginnen bei 0)
; Achtung: ECluster_SQLite_SelectRow() muss vor diesem Befehl ausgefhrt werden
;
; ECluster_SQLite: Datenbank - Handle
; Column: Spalte
;
; Rckgabe: Spalteninhalt
; ------------------------------------------------------------------------------------

Prototype.q _ECluster_SQLite_GetAsQuad(ECluster_SQLite, Column)

; ------------------------------------------------------------------------------------
; Gibt den Wert der Spalte Column als Double zurck. (Spalten beginnen bei 0)
; Achtung: ECluster_SQLite_SelectRow() muss vor diesem Befehl ausgefhrt werden
;
; ECluster_SQLite: Datenbank - Handle
; Column: Spalte
;
; Rckgabe: Spalteninhalt
; ------------------------------------------------------------------------------------

Prototype.d _ECluster_SQLite_GetAsDouble(ECluster_SQLite, Column)

; ------------------------------------------------------------------------------------
; Gibt den Wert der Spalte Column als BlobPointer zurck. (Spalten beginnen bei 0)
; Dieser Pointer muss spter mit ECluster_FreeMemory() freigegeben werden.
; Achtung: ECluster_SQLite_SelectRow() muss vor diesem Befehl ausgefhrt werden
;
; ECluster_SQLite: Datenbank - Handle
; Column: Spalte
;
; Rckgabe: Blobpointer
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_GetAsBlobPointer(ECluster_SQLite, Column)

; ------------------------------------------------------------------------------------
; Gibt die Gre des Blobs der Spalte Column zurck. (Spalten beginnen bei 0)
; Achtung: ECluster_SQLite_SelectRow() muss vor diesem Befehl ausgefhrt werden
;
; ECluster_SQLite: Datenbank - Handle
; Column: Spalte
;
; Rckgabe: Blobgre
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_GetAsBlobSize(ECluster_SQLite, Column)

; ------------------------------------------------------------------------------------
; Gibt die BlobFlags der Spalte Column zurck. (Spalten beginnen bei 0)
; Achtung: ECluster_SQLite_SelectRow() muss vor diesem Befehl ausgefhrt werden
;
; ECluster_SQLite: Datenbank - Handle
; Column: Spalte
;
; Rckgabe: Blobflags
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_GetAsBlobFlags(ECluster_SQLite, Column)

; ------------------------------------------------------------------------------------
; Gibt die Anzahl der Spalten zurck.
; Achtung: ECluster_SQLite_SelectRow() muss vor diesem Befehl ausgefhrt werden
;
; ECluster_SQLite: Datenbank - Handle
;
; Rckgabe: Spaltenanzahl
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_GetNumColumns(ECluster_SQLite)

; ------------------------------------------------------------------------------------
; Gibt den Typ der Spalte Column als String zurck. (Spalten beginnen bei 0)
; Achtung: ECluster_SQLite_SelectRow() muss vor diesem Befehl ausgefhrt werden
;
; ECluster_SQLite: Datenbank - Handle
; Column: Spalte
;
; Rckgabe: Spaltentyp
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_GetColumnTypeAsString(ECluster_SQLite, Column)

; ------------------------------------------------------------------------------------
; Gibt den Namen der Spalte Column zurck. (Spalten beginnen bei 0)
; Achtung: ECluster_SQLite_SelectRow() muss vor diesem Befehl ausgefhrt werden
;
; ECluster_SQLite: Datenbank - Handle
; Column: Spalte
;
; Rckgabe: Spaltenname
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_GetColumnName(ECluster_SQLite, Column)

; ------------------------------------------------------------------------------------
; Speicher eine Reihe in der Datenbank. Muss nachdem ECluster_SQLite_StoreAs...()
; Befehle ausgefhrt wurden aufgerufen werden, um die nderungen in die Datenbank
; zu schreiben.
;
; ECluster_SQLite: Datenbank - Handle
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_StoreRow(ECluster_SQLite)

; ------------------------------------------------------------------------------------
; Schreibt einen Long - Wert in die Spalte Column der Datenbank. (Spalten beginnen bei 0)
; Achtung: Nach alle ECluster_SQLite_StoreAs...() Befehlen muss ECluster_SQLite_StoreRow()
; ausgefhrt werden um die nderungen zu speichern.
;
; ECluster_SQLite: Datenbank - Handle
; Column: Spalte
; Value: Wert
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_StoreAsLong(ECluster_SQLite, Column, Value.l)

; ------------------------------------------------------------------------------------
; Schreibt einen Quad - Wert in die Spalte Column der Datenbank. (Spalten beginnen bei 0)
; Achtung: Nach alle ECluster_SQLite_StoreAs...() Befehlen muss ECluster_SQLite_StoreRow()
; ausgefhrt werden um die nderungen zu speichern.
;
; ECluster_SQLite: Datenbank - Handle
; Column: Spalte
; Value: Wert
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_StoreAsQuad(ECluster_SQLite, Column, Value.q)

; ------------------------------------------------------------------------------------
; Schreibt einen Double - Wert in die Spalte Column der Datenbank. (Spalten beginnen bei 0)
; Achtung: Nach alle ECluster_SQLite_StoreAs...() Befehlen muss ECluster_SQLite_StoreRow()
; ausgefhrt werden um die nderungen zu speichern.
;
; ECluster_SQLite: Datenbank - Handle
; Column: Spalte
; Value: Wert
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_StoreAsDouble(ECluster_SQLite, Column, Value.d)

; ------------------------------------------------------------------------------------
; Schreibt einen String - Wert in die Spalte Column der Datenbank. (Spalten beginnen bei 0)
; Achtung: Nach alle ECluster_SQLite_StoreAs...() Befehlen muss ECluster_SQLite_StoreRow()
; ausgefhrt werden um die nderungen zu speichern.
;
; ECluster_SQLite: Datenbank - Handle
; Column: Spalte
; Value: Wert
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_StoreAsString(ECluster_SQLite, Column, Text.s)

; ------------------------------------------------------------------------------------
; Schreibt einen Blob in die Spalte Column der Datenbank. (Spalten beginnen bei 0)
; Achtung: Nach alle ECluster_SQLite_StoreAs...() Befehlen muss ECluster_SQLite_StoreRow()
; ausgefhrt werden um die nderungen zu speichern.
;
; ECluster_SQLite: Datenbank - Handle
; Column: Spalte
; *Pointer: Speicheraddresse
; Size: Gre des Speichers
; Flags: Blobflags
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_StoreAsBlob(ECluster_SQLite, Column, *Pointer, Size, Flags = #ECluster_SQLite_BLOB_DEFAULT)

; ------------------------------------------------------------------------------------
; Gibt die letzte Fehlermeldung aus
;
; Rckgabe: Fehlermeldung
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_GetLastErrorString()

; ------------------------------------------------------------------------------------
; Setzt die Kompressionsstrke fr komprimierte Blobs
;
; ECluster_SQLite: Datenbank - Handle
; CompressionLevel: Kompressionsstrke (0-9)
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_SetCompressionLevel(ECluster_SQLite, CompressionLevel = 9)

; ------------------------------------------------------------------------------------
; Stellt sicher das alle nderungen wirklich in die Datenbank geschrieben wurden.
;
; ECluster_SQLite: Datenbank - Handle
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_Flush(ECluster_SQLite)

; ------------------------------------------------------------------------------------
; Gibt die aktuelle SQLite - Version zurck
;
; Rckgabe: SQLite - Version
; ------------------------------------------------------------------------------------

Prototype _ECluster_SQLite_GetSQLiteVersion()

; ------------------------------------------------------------------------------------
;- Funktionen Deklarieren
; ------------------------------------------------------------------------------------

Global ECluster_SQLite_Create._ECluster_SQLite_Create
Global ECluster_SQLite_Open._ECluster_SQLite_Open
Global ECluster_SQLite_Close._ECluster_SQLite_Close
Global ECluster_SQLite_Update._ECluster_SQLite_Update
Global ECluster_SQLite_UpdateSync._ECluster_SQLite_UpdateSync
Global ECluster_SQLite_Query._ECluster_SQLite_Query
Global ECluster_SQLite_EndQuery._ECluster_SQLite_EndQuery
Global ECluster_SQLite_SelectRow._ECluster_SQLite_SelectRow
Global _ECluster_SQLite_GetAsString._ECluster_SQLite_GetAsString
Global ECluster_SQLite_GetAsLong._ECluster_SQLite_GetAsLong
Global ECluster_SQLite_GetAsQuad._ECluster_SQLite_GetAsQuad
Global ECluster_SQLite_GetAsDouble._ECluster_SQLite_GetAsDouble
Global ECluster_SQLite_GetAsBlobPointer._ECluster_SQLite_GetAsBlobPointer
Global ECluster_SQLite_GetAsBlobSize._ECluster_SQLite_GetAsBlobSize
Global ECluster_SQLite_GetAsBlobFlags._ECluster_SQLite_GetAsBlobFlags
Global ECluster_SQLite_GetNumColumns._ECluster_SQLite_GetNumColumns
Global _ECluster_SQLite_GetColumnTypeAsString._ECluster_SQLite_GetColumnTypeAsString
Global _ECluster_SQLite_GetColumnName._ECluster_SQLite_GetColumnName
Global ECluster_SQLite_StoreRow._ECluster_SQLite_StoreRow
Global ECluster_SQLite_StoreAsLong._ECluster_SQLite_StoreAsLong
Global ECluster_SQLite_StoreAsQuad._ECluster_SQLite_StoreAsQuad
Global ECluster_SQLite_StoreAsDouble._ECluster_SQLite_StoreAsDouble
Global ECluster_SQLite_StoreAsString._ECluster_SQLite_StoreAsString
Global ECluster_SQLite_StoreAsBlob._ECluster_SQLite_StoreAsBlob
Global _ECluster_SQLite_GetLastErrorString._ECluster_SQLite_GetLastErrorString
Global ECluster_SQLite_SetCompressionLevel._ECluster_SQLite_SetCompressionLevel
Global ECluster_SQLite_Flush._ECluster_SQLite_Flush
Global _ECluster_SQLite_GetSQLiteVersion._ECluster_SQLite_GetSQLiteVersion

Procedure.s ECluster_SQLite_GetAsString(_DB, _Col)
  Protected String$, *StringMem = _ECluster_SQLite_GetAsString(_DB, _Col)
  If *StringMem
    String$ = PeekS(*StringMem)
    ECluster_FreeMemory(*StringMem)
    ProcedureReturn String$
  EndIf
EndProcedure

Procedure.s ECluster_SQLite_GetColumnTypeAsString(_DB, _Col)
  Protected String$, *StringMem = _ECluster_SQLite_GetColumnTypeAsString(_DB, _Col)
  If *StringMem
    String$ = PeekS(*StringMem)
    ECluster_FreeMemory(*StringMem)
    ProcedureReturn String$
  EndIf
EndProcedure

Procedure.s ECluster_SQLite_GetColumnName(_DB, _Col)
  Protected String$, *StringMem = _ECluster_SQLite_GetColumnName(_DB, _Col)
  If *StringMem
    String$ = PeekS(*StringMem)
    ECluster_FreeMemory(*StringMem)
    ProcedureReturn String$
  EndIf
EndProcedure

Procedure.s ECluster_SQLite_GetLastErrorString()
  Protected String$, *StringMem = _ECluster_SQLite_GetLastErrorString()
  If *StringMem
    String$ = PeekS(*StringMem)
    ECluster_FreeMemory(*StringMem)
    ProcedureReturn String$
  EndIf
EndProcedure

Procedure.s ECluster_SQLite_GetSQLiteVersion()
  Protected String$, *StringMem = _ECluster_SQLite_GetSQLiteVersion()
  If *StringMem
    String$ = PeekS(*StringMem)
    ECluster_FreeMemory(*StringMem)
    ProcedureReturn String$
  EndIf
EndProcedure

; IDE Options = PureBasic 4.31 (Windows - x86)
; CursorPosition = 426
; FirstLine = 390
; Folding = -
; EnableXP