; ------------------------------------------------------------------------------------
; Slave Routinen
; Serverteil von ECluster
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
;- Strukturen
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; Stream - Objekt
; ------------------------------------------------------------------------------------

Structure ECluster_Slave_StreamObject ; Stream - Objekt
  *StreamMem ; StreamMemory
  StreamMemSize.l ; Gesamte StreamMemory Gre
  CurrentStreamMemSize.l ; Aktuelle StreamMemoryGre
  StreamStatus.l ; StreamStatus
  StreamCancel.l ; Stream abbrechen
EndStructure

; ------------------------------------------------------------------------------------
;- Enumerationen
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; Stream Statusmglichkeiten
; ------------------------------------------------------------------------------------

Enumeration 1
  #Stream_InProgress = $01 ; Stream beim bertragen
  #Stream_Finished = $02; Stream Fertig
  #Stream_Error = $03; Fehler, Stream abgebrochen
  #Stream_Canceled = $04 ; Stream abgebrochen
EndEnumeration

; ------------------------------------------------------------------------------------
; Mgliche Rckgabewerte von ECluster_Slave_Event_Type()
; ------------------------------------------------------------------------------------

Enumeration 1
  ; Master Events (01 - EF)
  #Event_Master_Disconnect = $01; Master beendet Verbindung
  #Event_Master_Connect = $02 ; Master Verbunden
  #Event_Master_GetJobInfos = $03 ; Master fordert Jobliste an
  #Event_Master_DoJob = $04 ; Master fordert Job an
  #Event_Master_CountJobs = $05 ; Master zhlt verbleibende Jobs im Queue
  #Event_Master_CountWorkers = $06 ; Master zhlt Worker am Slave
  #Event_Master_UnknownCommand = $07 ; Unbekanntes Kommando
  #Event_Master_Ping = $08 ; Master Pingt Client
  
  ; Sonstiges (F0 - FF)
  #Event_Slave_BufferUnderrun = #Event_Slave_BufferUnderrun ; Buffer Unterlauf
EndEnumeration

; ------------------------------------------------------------------------------------
; LogTypen fr die Statistik
; ------------------------------------------------------------------------------------

Enumeration 1
  #Statistics_Events = $01 ; Events loggen
  #Statistics_Jobs = $02 ; Jobs loggen
  #Statistics_Masters = $04 ; Master loggen
  #Statistics_Default = #Statistics_Jobs | #Statistics_Masters
  #Statistics_All = #Statistics_Events | #Statistics_Jobs | #Statistics_Masters
EndEnumeration

; ------------------------------------------------------------------------------------
;- Callbacks
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; Von ECluster aufgerufenen Proceduren mssen folgendes Format haben
;
; ECluster_Slave: Slave Handle das die Procedure aufruft
; Master Number: Master Nummer fr die der Job ausgefhrt wird
; JobNumber: Aktuelle Job - Nummer
; ParameterStack: Stack mit den bergabe - Parametern oder 0 wenn keine Parameter bergeben wurden
; *StreamObject: 0 Wenn kein Stream, Sonst Pointer auf ein Stream - Object
;
; Mgliche Rckgabewerte:
; 0 oder die ECluster_Return_*() Funktionen benutzen
; ------------------------------------------------------------------------------------

Prototype ECluster_Job(ECluster_Slave, MasterNumber, JobNumber, ParameterStack, *StreamObject.ECluster_Slave_StreamObject)

PrototypeC ECluster_Job_C(ECluster_Slave, MasterNumber, JobNumber, ParameterStack, *StreamObject.ECluster_Slave_StreamObject)

; ------------------------------------------------------------------------------------
; Callback fr ECluster_Slave_IterateMasters()
;
; ECluster_Slave: Slave Handle
; MasterNumber: aktuelle Master Nummer
; UserData: angegebene UserData
;
; Rckgabe: <> #False: Iteration fortsetzen, #False: Iteration abbrecheb
; ------------------------------------------------------------------------------------

Prototype ECluster_Slave_MasterCB(ECluster_Slave, MasterNumber, UserData)

; ------------------------------------------------------------------------------------
; Callback fr Slave - Events
;
; ECluster_Slave: Slave Handle
; MasterNumber: aktuelle Master Nummer
; Job$: Job Name
; EventType: EventTyp (wie ECluster_Slave_Event_Type())
; UserData: angegebene UserData
;
; Rckgabe: <> #False: Iteration fortsetzen, #False: Iteration abbrechen
; ------------------------------------------------------------------------------------

Prototype ECluster_Slave_EventCB(ECluster_Slave, MasterNumber, Job$, EventType, UserData)

; ------------------------------------------------------------------------------------
; Callback fr Zugriffscheck bei Jobs
;
; ECluster_Slave: Slave Handle
; MasterNumber: aktuelle Master Nummer
; Job: Job Name
;
; Rckgabe: <> #False: Job ausfhren, #False: Job verweigern
; ------------------------------------------------------------------------------------

Prototype ECluster_Slave_JobAccessCheck(ECluster_Slave, MasterNumber, Job$)

; ------------------------------------------------------------------------------------
;- Slave Routinen
;
; Routinen zur Steuerung eines Slave
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; Initialisiert einen neuen Slave
;
; Port: Port an dem der Slave luft
; NumberOfThreads: Anzahl der Worker - Threads des Slave
;
; Rckgabe: Slave - Handle wenn Ok, #False wenn Fehler
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Init(Port, NumberOfThreads = 1)

; ------------------------------------------------------------------------------------
; Beendet einen Slave
;
; ECluster_Slave: Slave Handle
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Deinit(ECluster_Slave)

; ------------------------------------------------------------------------------------
; Beschreibung des letzen Fehlers
;
; ECluster_Slave: Slave Handle
;
; Rckgabe: Fehlermeldung
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_GetLastError(ECluster_Slave)

; ------------------------------------------------------------------------------------
; Pausiert die Abarbeitung der JobQueue
;
; ECluster_Slave: Slave Handle
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Pause(ECluster_Slave)

; ------------------------------------------------------------------------------------
; Setzt die Abarbeitung der JobQueue fort
;
; ECluster_Slave: Slave Handle
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Resume(ECluster_Slave)

; ------------------------------------------------------------------------------------
; Gibt ein Stream-Objekt frei
;
; ECluster_Slave: Slave Handle
; *Stream: Stream Objekt
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_FreeStream(ECluster_Slave, *Stream)

; ------------------------------------------------------------------------------------
; Zhlt die Anzahl der Masters am Slave
;
; ECluster_Slave: Slave Handle
;
; Rckgabe: MasterCount
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_CountMasters(ECluster_Slave)

; ------------------------------------------------------------------------------------
; Iteriert die Masters des Slave und sendet sie an ein CallBack
;
; ECluster_Slave: Slave Handle
; *CallBack: Pointer auf ECluster_Slave_MasterCB Procedure
; UserData: Individuelle Daten
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_IterateMasters(ECluster_Slave, *CallBack.ECluster_Slave_MasterCB, UserData)

; ------------------------------------------------------------------------------------
; Kickt einen Master vom Slave
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_KickMaster(ECluster_Slave, Master)

; ------------------------------------------------------------------------------------
; Setzt die das Job - Limit fr einen/alle Master
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer (0 = Alle Master)
; Limit: Job Limit (0 : kein Limit, <=-1 : Keine Jobs)
;
; Rckgabe: #True
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_SetMasterJobLimit(ECluster_Slave, Master, Limit)

; ------------------------------------------------------------------------------------
; (De)aktiviert die P2P - Streambertragung fr einen/alle Master
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer (0 = Alle Master)
; State: Status (#False: Deaktiviert, #True: Aktiviert) (Default: #True)
;
; Rckgabe: #True
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_EnableStreamTransmission(ECluster_Slave, Master, State)

; ------------------------------------------------------------------------------------
; Setzt Callback fr den Zugangscheck fr einen/alle Master
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer (0 = Alle Master)
; *CallBack: CallBack (siehe ECluster_Slave_JobAccessCheck)
;
; Rckgabe: #True
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_SetMasterJobAccessCheck(ECluster_Slave, Master, *Callback)

; ------------------------------------------------------------------------------------
; Ermittelt die Anzahl austehender Jobs fr einen Master
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer
;
; Rckgabe: aktuelle Job-Anzahl, -1 bei Fehler
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_CountMasterJobs(ECluster_Slave, Master)

; ------------------------------------------------------------------------------------
; Ermittelt die IP des Masters (zur Verarbeitung mit IPString())
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer
;
; Rckgabe: MasterIP
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_GetMasterIP(ECluster_Slave, Master)

; ------------------------------------------------------------------------------------
; Ermittelt den Port des Masters
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer
;
; Rckgabe: MasterPort
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_GetMasterPort(ECluster_Slave, Master)

; ------------------------------------------------------------------------------------
; Holt die gespeicherten Xtra-Daten des Master
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer
;
; Rckgabe: Xtra - Data des Master
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_GetMasterXtraData(ECluster_Slave, Master)

; ------------------------------------------------------------------------------------
; Setzt die Xtra-Daten des Master
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer
; XtraData: XtraDaten
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_SetMasterXtraData(ECluster_Slave, Master, XtraData)

; ------------------------------------------------------------------------------------
; Ermittelt die Anzahl der ausstehenden Jobs
;
; ECluster_Slave: Slave Handle
;
; Rckgabe: Austehende Jobs
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_CountJobs(ECluster_Slave)

; ------------------------------------------------------------------------------------
; Setzt die EmpfangsBlock - Puffergre
;
; ECluster_Slave: Slave Handle
; Size: neue EmpfangsBlock - Puffergre in Bytes > 0 (Default: 512 Kb)
;
; Rckgabe: Alte Puffergre, -1 bei Fehler
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_SetReceiveBlockSize(ECluster_Slave, Size = 64*1024)

; ------------------------------------------------------------------------------------
; Setzt die Einstellungen fr die Automatische Kompression
;
; ECluster_Slave: Slave Handle
; Limit: Limit ab dem Komprimiert werden soll (0: deaktiviert, <0: alles komrimieren)
; Level: Kompressions-Level (1-9, 0 = deaktiviert)
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_SetCompressionOptions(ECluster_Slave, Limit, Level)

; ------------------------------------------------------------------------------------
; Setzt die Einstellungen fr ein globales Callback
;
; ECluster_Slave: Slave Handle
; *CallBack: CallBack (siehe ECluster_Slave_EventCB)
; UserData: UserData fr Callback
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_SetCallback(ECluster_Slave, *CallBack, UserData)

; ------------------------------------------------------------------------------------
; Aktiviert den Thread Empfangsmodus des Slave
;
; ECluster_Slave: Slave Handle
; State: <> 0: Aktiviert, 0: Deaktiviert
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_ThreadReceive(ECluster_Slave, State)

; ------------------------------------------------------------------------------------
; Aktiviert die Statistik - Funktionen des Slave
;
; ECluster_Slave: Slave Handle
; DataBase$: Datenbank zum speichern (SQLite)
; LogType: Was soll gelogt werden?
;
; Rckgabe: #True falls erfolgreich
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_EnableStatistics(ECluster_Slave, DataBase$, LogType = #Statistics_Default)

; ------------------------------------------------------------------------------------
; Fgt einen verfgbaren Job zum Slave hinzu
;
; ECluster_Slave: Slave Handle
; JobName$: Name des Job
; JobDescription$: Job Beschreibung
; JobProcedure: Pointer auf eine ECluster_Job() Procedure
; cdecl: Verwendet die cdecl - Aufrufkonvention fr den Job (Standard: stdcall)
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_AddJob(ECluster_Slave, JobName$, JobDescription$, JobProcedure.ECluster_Job, cdecl = 0)

; ------------------------------------------------------------------------------------
; Schaltet Events auf dem Slave ein/aus
;
; ECluster_Slave: Slave Handle
; Enable: <> 0: Events ein, 0: Events aus
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_LogEvents(ECluster_Slave, Enable)

; ------------------------------------------------------------------------------------
; Prft ob Events vorhanden sind
;
; ECluster_Slave: Slave Handle
;
; Rckgabe: Event-Anzahl wenn vorhanden, sonst #False
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Event(ECluster_Slave)

; ------------------------------------------------------------------------------------
; Typ des aktuellen Events
;
; ECluster_Slave: Slave Handle
;
; Rckgabe: EventType
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Event_Type(ECluster_Slave)

; ------------------------------------------------------------------------------------
; Master des aktuellen Events
;
; ECluster_Slave: Slave Handle
;
; Rckgabe: EventMaster
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Event_Master(ECluster_Slave)

; ------------------------------------------------------------------------------------
; JobName des aktuellen Events
;
; ECluster_Slave: Slave Handle
;
; Rckgabe: JobName
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Event_Job(ECluster_Slave)

; ------------------------------------------------------------------------------------
; Sendet Benutzerdefinierte Daten an den Master
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer
; JobNumber: Job Nummer
; *Mem: Speicherblock
; MemSize: Speichergre
;
; Rckgabe: #Null
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Job_SendMessage(ECluster_Slave, Master, JobNumber, *Mem, MemSize)

; ------------------------------------------------------------------------------------
;- Funktionen Deklarieren
; ------------------------------------------------------------------------------------

Global ECluster_Slave_Init._ECluster_Slave_Init
Global ECluster_Slave_DeInit._ECluster_Slave_DeInit
Global _ECluster_Slave_GetLastError._ECluster_Slave_GetLastError
Global ECluster_Slave_Pause._ECluster_Slave_Pause
Global ECluster_Slave_Resume._ECluster_Slave_Pause
Global ECluster_Slave_FreeStream._ECluster_Slave_FreeStream
Global ECluster_Slave_CountMasters._ECluster_Slave_CountMasters
Global ECluster_Slave_IterateMasters._ECluster_Slave_IterateMasters
Global ECluster_Slave_KickMaster._ECluster_Slave_KickMaster
Global ECluster_Slave_SetMasterJobLimit._ECluster_Slave_SetMasterJobLimit
Global ECluster_Slave_EnableStreamTransmission._ECluster_Slave_EnableStreamTransmission
Global ECluster_Slave_CountMasterJobs._ECluster_Slave_CountMasterJobs
Global ECluster_Slave_GetMasterIP._ECluster_Slave_GetMasterIP
Global ECluster_Slave_GetMasterPort._ECluster_Slave_GetMasterPort
Global ECluster_Slave_GetMasterXtraData._ECluster_Slave_GetMasterXtraData
Global ECluster_Slave_SetMasterXtraData._ECluster_Slave_SetMasterXtraData
Global ECluster_Slave_SetMasterJobAccessCheck._ECluster_Slave_SetMasterJobAccessCheck
Global ECluster_Slave_CountJobs._ECluster_Slave_CountJobs
Global ECluster_Slave_SetReceiveBlockSize._ECluster_Slave_SetReceiveBlockSize
Global ECluster_Slave_SetCompressionOptions._ECluster_Slave_SetCompressionOptions
Global ECluster_Slave_SetCallback._ECluster_Slave_SetCallback
Global ECluster_Slave_ThreadReceive._ECluster_Slave_ThreadReceive
Global ECluster_Slave_EnableStatistics._ECluster_Slave_EnableStatistics
Global ECluster_Slave_AddJob._ECluster_Slave_AddJob
Global ECluster_Slave_LogEvents._ECluster_Slave_LogEvents
Global ECluster_Slave_Event._ECluster_Slave_Event
Global ECluster_Slave_Event_Type._ECluster_Slave_Event_Type
Global ECluster_Slave_Event_Master._ECluster_Slave_Event_Master
Global _ECluster_Slave_Event_Job._ECluster_Slave_Event_Job
Global ECluster_Slave_Job_SendMessage._ECluster_Slave_Job_SendMessage

Procedure.s ECluster_Slave_Event_Job(ECluster_Slave)
  Protected *StringMem = _ECluster_Slave_Event_Job(ECluster_Slave)
  If *StringMem
    ProcedureReturn PeekS(*StringMem)
  EndIf
EndProcedure

Procedure.s ECluster_Slave_GetLastError(ECluster_Slave)
Protected *StringMem = _ECluster_Slave_GetLastError(ECluster_Slave)
If *StringMem
  ProcedureReturn PeekS(*StringMem)
EndIf
EndProcedure


; IDE Options = PureBasic 4.31 (Windows - x86)
; CursorPosition = 480
; FirstLine = 458
; Folding = -
; EnableXP